"""This addon is a bridge between Blender and Ragdoll"""

import os
import sys
import platform

bl_info = {
    "name": "Ragdoll",
    "version": (2024, 2, 1),      # Auto generated. DO NOT TOUCH!
    "blender": (3, 4, 0),         # The minimum Blender version required
    "ragdollcore": (2024, 2, 1),  # The minimum Ragdoll Core version required
    "category": "Animation",
    "author": "Ragdoll Dynamics",
    "description": "Animate with real-time physics.",
    "location": "Physics Properties",
    "doc_url": "https://learn.ragdolldynamics.com/overview/",
    "tracker_url": "https://forums.ragdolldynamics.com/",
}


platform_name = platform.system()
core_version = ".".join("%02d" % i for i in bl_info["ragdollcore"])


def get_install_dir():
    if platform_name == "Windows":
        root = os.getenv("PROGRAMFILES")

    elif platform_name == "Darwin":
        root = "Not yet implemented"

    elif platform_name == "Linux":
        root = "Not yet implemented"

    else:
        # Cannot raise exception from init
        root = "Unsupported OS: %s" % platform_name

    path = os.path.join(root, "Ragdoll Dynamics", "Core", core_version, "lib")

    # Enable override via environment variable
    path = os.getenv("RAGDOLL_CORE_PATH", path)

    return path


# Expose .pyd extension
root = get_install_dir()

# Enable the user to report where Core was attempted
print("Looking for Ragdoll Core @ '%s'" % root)
dll = os.path.join(root, "%sragdollcore.%s" % ("", "dll"))


if os.path.exists(dll):
    sys.path.insert(0, root)

    def register():
        """When the add-on checkbox checked, could be on Blender startup"""

        from . import main
        main.install()

    def unregister():
        """When the add-on checkbox unchecked"""
        from . import main
        main.uninstall()

# Help the user download and install the missing Ragdoll SDK or Standalone
else:
    def register():
        """When the add-on checkbox checked, could be on Blender startup"""
        from . import placeholder
        placeholder.install(platform_name, core_version)

    def unregister():
        """When the add-on checkbox unchecked"""
        from . import placeholder
        placeholder.uninstall(platform_name, core_version)
