import bpy

from . import OperatorWithOptions
from .. import scene, util
from ..vendor import bpx


class CreatePinConstraint(OperatorWithOptions):
    bl_idname = "ragdoll.create_pin_constraint"
    bl_label = "Create Pin Constraint"
    bl_options = {"REGISTER", "UNDO"}
    bl_description = "Create a new solver"

    def execute(self, context):
        solver = scene.find_or_create_current_solver()

        if not solver:
            self.report({"ERROR"}, "No solver found")
            return {"CANCELLED"}

        selection = bpx.selection()
        entities = list(filter(None, [
            s.data.get("entity") for s in selection
        ]))

        if not any(entities):
            self.report({"ERROR"}, "No markers selected")
            return {"CANCELLED"}

        new_constraints = []
        for entity in entities:
            xobj = bpx.alias(entity, None)

            # An entity without an object? Very rare
            if not xobj:
                self.report({"WARNING"}, "No object for entity %s" % entity)
                continue

            if xobj.type() != "rdMarker":
                self.report({"WARNING"}, "%s was not a Marker" % xobj)
                continue

            xcon = scene.create("rdPinConstraint", name="rPinConstraint")
            xcon["childMarker"] = xobj.handle()

            # More suitable default values
            xcon["linearStiffness"] = 0.01
            xcon["angularStiffness"] = 0.01

            # TODO: Make dependent on option
            transform = xobj["sourceTransform"].read()
            xcon.handle().matrix_world = transform.matrix()

            # Append to solver
            solver["members"].append({"object": xcon.handle()})
            new_constraints.append(xcon)
            bpx.link(xcon, util.find_assembly())

        with bpx.object_mode():
            bpx.select(new_constraints)

        util.touch_initial_state(context)

        return {"FINISHED"}


def install():
    bpy.utils.register_class(CreatePinConstraint)


def uninstall():
    bpy.utils.unregister_class(CreatePinConstraint)
